@@ -1,11 +1,38 @@
 from functools import wraps
-import os
 
-import colorama
-from colorama import Fore, Back, Style
-colorama.init()
+import rich.console
+import rich.theme
+import rich.panel
+import rich.text
+import rich.progress
+import rich.traceback
+import rich.markdown
 
-__version__ = "2.8.5"
+__version__ = "3.3.1"
+theme = rich.theme.Theme({
+    "echo": "on black",
+    "echo-bar": "on black",
+    "echo-pre": "dim on black",
+    "info": "",
+    "info-bar": "",
+    "info-pre": "dim",
+    "warn": "red",
+    "warn-bar": "red",
+    "warn-pre": "dim red",
+    "err": "bright_white on red",
+    "err-bar": "bright_red",
+    "err-pre": "dim bright_red",
+    "ask": "yellow",
+    "ask-bar": "yellow",
+    "ask-pre": "dim yellow",
+    "blur": "dim bright_white",
+    "blur-bar": "dim white",
+    "blur-pre": "dim white",
+    "title": "bright_cyan",
+    "pause": "yellow",
+})
+console = rich.console.Console(theme=theme)  # main output printer
+rich.traceback.install(show_locals=True)
 
 
 def as_session(name_or_func):  # decorator
@@ -27,7 +54,7 @@ def as_session(name_or_func):  # decorator
         @wraps(func)
         def wrapper(*args, **kwargs):
             start()
-            title(name)
+            title(f"{name}()")
             result = func(*args, **kwargs)
             end()
             return result
@@ -36,11 +63,11 @@ def as_session(name_or_func):  # decorator
 
 
 def start():
-    print('*')
+    br()
 
 
 def end():
-    print('`')
+    console.print("╰")
 
 
 def br(count=1):
@@ -48,48 +75,64 @@ def br(count=1):
     print('\n' * (count - 1))
 
 
-def echo(msg, pre=""):
-    prefix = Style.DIM + Fore.WHITE + '({}) '.format(pre.capitalize()) + Fore.RESET + Style.RESET_ALL if pre else ''
-    print("| " + Back.BLACK + "{pf}{msg}".format(pf=prefix, msg=msg) + Back.RESET + Fore.RESET + Style.RESET_ALL)
+def echo(*args, pre: str = "", bar: str = "│", style: str = "echo", **options):
+    txt = rich.text.Text()
+    if bar:
+        txt.append(f"{bar}", style=f"{style}-bar")
+        txt.append(" ")
+    if pre:
+        txt.append(f"({pre.capitalize()})", style=f"{style}-pre")
+        txt.append(" ")
+    txt.append(" ".join([f"{arg}" for arg in args]), style=style)
+    console.print(txt.markup, **options)
 
 
-def title(msg, **options):
+def title(*args, **options):
     """print something like a title"""
-    return echo(Style.BRIGHT + Fore.CYAN + "__{}__________________________".format(msg.upper().strip()) + Style.RESET_ALL + Fore.RESET, **options)
+    return console.print(rich.panel.Panel((" ".join([f"{arg}" for arg in args])).upper().strip(), highlight=True, expand=False), **options)
 
 
-def ask(msg, **options):
-    return echo(Fore.YELLOW + msg, "?", **options)
+def ask(*args, **options):
+    return echo(*args, pre="?", style="ask", **options)
 
 
-def info(msg, **options):
-    return echo(msg, "info", **options)
+def info(*args, **options):
+    return echo(*args, pre="info", style="info", **options)
 
 
-def warn(msg, **options):
-    return echo(Fore.RED + msg, "warning", **options)
+def warn(*args, **options):
+    return echo(*args, pre="warning", style="warn", **options)
 
 
-def err(msg, **options):
-    return echo(Back.RED + Fore.WHITE + Style.BRIGHT + msg, "error", **options)
+def err(*args, **options):
+    return echo(*args, pre="error", style="err", **options)
 
 
-def dim(msg, **options):
-    return echo(Style.DIM + Fore.WHITE + msg, **options)
+def dim(*args, **options):
+    return echo(*args, style="blur", **options)
 
 
-def pause(msg="Press Enter to Continue..."):
+def print(*args, **options):
+    console.print(*args, **options)
+
+
+def markdown(*args, **options):
+    console.print(rich.markdown.Markdown(" ".join([f"{arg}" for arg in args])))
+
+
+def pause(msg="Press [Enter] to Continue..."):
     """press to continue"""
-    print('\n' + Fore.YELLOW + msg + Fore.RESET, end='')
-    input()
+    br()
+    echo(f"{rich.markup.escape(msg)}", bar="", style="pause", end="")
+    return input()
 
 
-def bye(msg=''):
+def bye(msg=""):
     """print msg and exit"""
     exit(msg)
 
 
-def get_input(question='', prompt='> '):
+def get_input(question="", prompt="> "):
     if question:
         ask(question)
     return str(input(prompt)).strip()
@@ -103,11 +146,11 @@ def get_choice(choices, exitable: bool = False):
         exitable: bool. Does `exit` is an option for user to select.
     """
     EXIT_WORD = "exit" if "0" in choices else "0"
+    fill = max(len(EXIT_WORD), 2)
     for index, item in enumerate(choices, start=1):
-        assemble_print = "{Fore.YELLOW}{num:>2}){Fore.RESET} {Fore.WHITE}{itm}{Fore.RESET}".format(Fore=Fore, num=index, itm=item)
-        echo(assemble_print)
+        console.print(f"  [yellow]{index:>{fill}})[/] [white]{item}[/]")
     if exitable:
-        echo("{Fore.YELLOW}{word:>2}) ** EXIT **{Fore.RESET}".format(Fore=Fore, word=EXIT_WORD))
+        console.print(f"  [yellow]{EXIT_WORD:>{fill}}) ** [b]EXIT[/] **")
     user_choice = get_input().strip()
     if exitable and user_choice == EXIT_WORD:
         return None
@@ -132,18 +175,18 @@ def get_choices(choices, allable: bool = False, exitable: bool = False) -> list:
     EXIT_WORD = "exit" if "0" in choices else "0"
     DONE_WORD = "done" if "0" in choices else "0"
     ALL_WORD = "all" if "a" in choices else "a"
+    fill = max(len(EXIT_WORD), len(DONE_WORD), len(ALL_WORD), 2)
     user_choices = []
     while True:
         if allable:
-            echo("{Fore.YELLOW}{word:>2}) ** ALL **{Fore.RESET}".format(Fore=Fore, word=ALL_WORD))
+            console.print(f"  [yellow]{ALL_WORD:>{fill}}) ** [b]ALL[/] **")
         for index, item in enumerate(choices, start=1):
-            mark = "[+]" if item in user_choices else "[ ]"  # item is selected or not
-            assemble_print = "{Fore.YELLOW}{num:>2}){Fore.RESET} {mark} {Fore.WHITE}{itm}{Fore.RESET}".format(Fore=Fore, num=index, itm=item, mark=mark)
-            echo(assemble_print)
+            mark = r"\[[green]+[/]]" if item in user_choices else rich.markup.escape("[ ]")  # item is selected or not
+            console.print(f"  [yellow]{index:>{fill}})[/] {mark} [white]{item}")
         if user_choices:  # user selections > 0
-            echo("{Fore.YELLOW}{word:>2}) ** DONE **{Fore.RESET}".format(Fore=Fore, word=DONE_WORD))
+            console.print(f"  [yellow]{DONE_WORD:>{fill}}) ** [b]DONE[/] **")
         elif exitable:  # no user selection, but exitable is on.
-            echo("{Fore.YELLOW}{word:>2}) ** EXIT **{Fore.RESET}".format(Fore=Fore, word=EXIT_WORD))
+            console.print(f"  [yellow]{EXIT_WORD:>{fill}}) ** [b]EXIT[/] **")
         user_choice = get_input().strip()
         if (user_choice == DONE_WORD or user_choice == EXIT_WORD):
             if exitable or len(user_choices) > 0:  # keep looping when not exitable and no user choices.
@@ -162,6 +205,16 @@ def get_choices(choices, allable: bool = False, exitable: bool = False) -> list:
             err("Please enter a valid choice.")
 
 
+def track(iterable, desc="", unit="", *args, **options):
+    with rich.progress.Progress("│", rich.progress.SpinnerColumn(), *rich.progress.Progress.get_default_columns(),
+                                "·", rich.progress.MofNCompleteColumn(), unit) as progress:
+        task = progress.add_task(desc, total=None if not iterable or not len(iterable) else len(iterable))
+        while not progress.finished:
+            for item in iterable:
+                yield item
+                progress.update(task, advance=1)
+
+
 def read_file(path: str, with_encoding: bool = False, **kwargs):
     for enc in ("utf-8", 'gbk', 'cp1252', 'windows-1252', 'latin-1'):
         try:
@@ -175,3 +228,7 @@ def write_file(path: str, content: str, overwrite: bool = False, **kwargs):
     mode = 'w' if overwrite else 'a'
     with open(path, mode=mode, encoding='utf-8', **kwargs) as fl:
         return fl.write(content)
+
+
+if __name__ == "__main__":
+    markdown(read_file("README.md"))
